

. .\Configuration.ps1
. .\CommonFunctions.ps1

# Search site full URL
$searchSite = $rootUrl + $searchSitePath

# Profile pages host URL
$profilePages = $rootUrl + $profilePagesPath

# Crawl rule path for QPR content
$crawlRulePath = "bdc3://qpr_model_*"

# Check that the target search application exists and get its 
# localized name for localized SharePoints
$targetSearchApp = VerifySearchApplication($targetSearchApp)

Write-Host -foregroundcolor Yellow "Checking QPR Search Central components..."

$searchApplication = Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp;
$crawlRule = Get-SPEnterpriseSearchCrawlRule -SearchApplication $searchApplication | where { ($_.Path -eq $crawlRulePath) -and ($_.Type -eq "InclusionRule") };

if ($crawlRule)
{
  Write-Host -foregroundcolor green "* Crawl rule exists"
} else
{
  Write-Host -foregroundcolor red "* Crawl rule does not exist";
}


$securityTrimmer = Get-SPEnterpriseSearchSecurityTrimmer -SearchApplication $searchApplication | where { ($_.Id -eq 1) -and ($_.CrawlRulePath -like "*" + $crawlRulePath + "*") -and ($_.TypeName -like "*Microsoft.Office.Server.Search.Connector.BDC.SPBDC.SPBdcSecurityTrimmer*") };
if ($securityTrimmer)
{
  Write-Host -foregroundcolor green "* Security trimmer exists"
} else
{
  Write-Host -foregroundcolor red "* Security trimmer does not exist";
}

$solution = Get-SPSolution | where { $_.Name -eq "Qpr.BdcModel.wsp" };
if ($solution)
{
  Write-Host -foregroundcolor green "* Qpr.BdcModel.wsp solution exists"
} else
{
  Write-Host -foregroundcolor red "* Qpr.BdcModel.wsp solution does not exist";
}

if ($solution.Deployed)
{
  Write-Host -foregroundcolor green "* Qpr.BdcModel.wsp solution is deployed"
} else
{
  Write-Host -foregroundcolor red "* Qpr.BdcModel.wsp solution is not deployed";
}

$solution = Get-SPSolution | where { $_.Name -eq "Qpr.MasterPage.wsp" };
if ($solution)
{
  Write-Host -foregroundcolor green "* Qpr.MasterPage.wsp solution exists"
} else
{
  Write-Host -foregroundcolor red "* Qpr.MasterPage.wsp solution does not exist";
}

if ($solution.Deployed)
{
  Write-Host -foregroundcolor green "* Qpr.MasterPage.wsp solution is deployed"
} else
{
  Write-Host -foregroundcolor red "* Qpr.MasterPage.wsp solution is not deployed";
}


$webAppsFeature = $(Get-SPFeature -limit all | where {$_.displayname -eq "QPR.BdcModel_BDC Feature"})
if ($webAppsFeature.Id)
{
  Write-Host -foregroundcolor green "* Qpr.BdcModel_BDC feature exists"
} else
{
  Write-Host -foregroundcolor red "* Qpr.BdcModel_BDC feature does not exist";
}

if ($webAppsFeature.Id)
{
  Write-Host -foregroundcolor blue "* * * Qpr.BdcModel_BDC feature status is" $webAppsFeature.status
} 

$webAppsFeature = $(Get-SPFeature -limit all | where {$_.displayname -eq "QPR.MasterPage_QPRSiteContents"})
if ($webAppsFeature.Id)
{
  Write-Host -foregroundcolor green "* QPR.MasterPage_QPRSiteContents feature exists"
} else
{
  Write-Host -foregroundcolor red "* QPR.MasterPage_QPRSiteContents feature does not exist";
}

if ($webAppsFeature.Id)
{
  Write-Host -foregroundcolor blue "* * * QPR.MasterPage_QPRSiteContents feature status is" $webAppsFeature.status
} 

$profilePageHost = Get-SPSite $profilePages -ErrorAction SilentlyContinue
if ($profilePageHost)
{
  Write-Host -foregroundcolor green "* Profile page host site exists"
} else
{
  Write-Host -foregroundcolor red "* Profile page host site does not exist"
}


$searchSiteDefined = Get-SPSite $searchSite -ErrorAction SilentlyContinue
if ($searchSiteDefined)
{
  Write-Host -foregroundcolor green "* Search site exists"
} else
{
  Write-Host -foregroundcolor red "* Search site does not exist"
}

$contentSource = Get-SPEnterpriseSearchCrawlContentSource -SearchApplication (Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp) -identity "QPR BDC Content Source" -ErrorAction SilentlyContinue
if ($contentSource)
{
  Write-Host -foregroundcolor green "* QPR BDC Content Source exists"

  $bdc = Get-SPBusinessDataCatalogMetaDataObject -BdcObjectType Catalog -ServiceContext $rootUrl
  $qprModel = $bdc.GetLobSystems("*") | Where-Object {$_.DefaultDisplayName -like "*Qpr.Model*"}
  Write-Host -foregroundcolor Blue "Following external content types are defined for the content source:"
  foreach ($entity in $qprModel.Entities)
  {
    Write-Host -foregroundcolor Blue "- "$entity.Name	  
  }

} else
{
  Write-Host -foregroundcolor red "* QPR BDC Content Source does not exist";
}

Write-Host -foregroundcolor Yellow "QPR Search Central component check complete"