function Load-Snapins()
{
    $snapinsToCheck = @("Microsoft.SharePoint.PowerShell")
    $currentSnapins = Get-PSSnapin

    $snapinsToCheck | ForEach-Object `
        {$snapin = $_;
            if(($CurrentSnapins | Where-Object {$_.Name -eq "$snapin"}) -eq $null)
            {
                Write-Host "$snapin snapin not found, loading it"
                Add-PSSnapin $snapin
                Write-Host "$snapin snapin loaded"
            }
        }
}

function AddAccountToAllSites($siteURL, $accountName, $permLevel, [switch]$skipRootSite, $newGroupDescription)
{
    #Get Site Collection
    $site = Get-SPSite $siteURL
   
    #Check if the accountName variable contains a slash - if so, it is an AD account
    #If not, it is a SharePoint Group
    $rootWeb = $site.RootWeb
    if ($accountName.Contains("\")) { $account = $rootWeb.EnsureUser($accountName) }
    else {
        #If the SharePoint Group does not exist, create it with the name and description specified
        if (!$rootWeb.SiteGroups[$accountName])
        {
            $rootWeb.SiteGroups.Add($accountName, $rootWeb.CurrentUser, $rootWeb.CurrentUser, $newGroupDescription)
        }
        $account = $rootWeb.SiteGroups[$accountName]
    }
    $rootWeb.Dispose()
   
    #Step through each site in the site collection
    $site | Get-SPWeb -limit all | ForEach-Object {
       
        #Check if the user has chosen to skip the root site - if so, do not change permissions on it
        if (($skipRootSite) -and ($site.Url -eq $_.Url)) { write-host "Root site" $_.Url "will be bypassed" }
        else {
            #Check if the current site is inheriting permissions from its parent
            #If not, set permissions on current site
            if ($_.HasUniqueRoleAssignments) {

                $assignment = New-Object Microsoft.SharePoint.SPRoleAssignment($account)
                if ($permLevel -eq "Read") {
                  #Someone in Redmond thought it would be a brilliant idea to localize also the permission levels. Let's work around that.
                  foreach ($roleDefinition in $_.RoleDefinitions)
                  {
                      if ($roleDefinition.type -eq "Reader")
                      {
                          $permLevel = $roleDefinition.Name
                      }
                  }
                }
                $role = $_.RoleDefinitions[$permLevel]
                $assignment.RoleDefinitionBindings.Add($role)
                $_.RoleAssignments.Add($assignment)

                write-host -foregroundcolor green "Account" $accountName "added to site" $_.Url "with" $permLevel "permissions."
            }
            else {
                write-host "Site" $_.Url "will not be modified as it inherits permissions from a parent site."
            }
        }
    }
    $site.Dispose()
}

function VerifySearchApplication($targetSearchApp)
{
  $searchApp = Get-SPServiceApplication -Name $targetSearchApp -ErrorAction SilentlyContinue
  if (!$searchApp) 
  {
    Write-Host Target search application not found by name, trying by typename
    $searchApp = Get-SPServiceApplication | Where {$_.TypeName -eq $targetSearchApp}
    if (!$searchApp)
    {
      Write-Host -foregroundcolor Red Search application not found! Verify that the defined search application exists. Installation will now terminate.
      exit
    }
    else 
    {
      Write-Host Switching target search app to $searchApp.Name
      $targetSearchApp = $searchApp.Name
      return $targetSearchApp
    }
  }
  else {
    return $targetSearchApp
  }
}