# Include common functions and configuration

. .\Configuration.ps1
. .\CommonFunctions.ps1
. .\InitializeQPRSearchSite.ps1

# Search site full URL
$searchSite = $rootUrl + $searchSitePath

# Profile pages host URL
$profilePages = $rootUrl + $profilePagesPath

# Crawl rule path for QPR content
$crawlRulePath = "bdc3://qpr_model_*"

# Control variable for prompts in ConfigureBdcModel.ps1
$inMainSetupScript = $true;
. .\ConfigureBdcModel.ps1

# Check that the target search application exists and get its 
# localized name for localized SharePoints
$targetSearchApp = VerifySearchApplication($targetSearchApp)

# Function definitions

Function CreateMetadataMappingIfNeeded([string]$managedPropertyName, [string]$crawledPropertyName, [int]$type, [int]$crawledType)
{
  $ErrorActionPreference = "SilentlyContinue"
  $cp = Get-SPEnterpriseSearchMetadataCrawledProperty -SearchApplication $searchapp -Category $category -Name $crawledPropertyName
  if (!$cp) {
    $cp = New-SPEnterpriseSearchMetadataCrawledProperty -SearchApplication $searchapp -Category $category -VariantType $crawledType -PropSet "2edeba9a-0fa8-4020-8a8b-30c3cdf34ccd" -Name $crawledPropertyName -IsNameEnum $false
    write-host "Created new crawled property named $($crawledPropertyName)."
  }

  $mp = Get-SPEnterpriseSearchMetadataManagedProperty -SearchApplication $searchapp -Identity $managedPropertyName
  if (-not $?) {
    $mp = New-SPEnterpriseSearchMetadataManagedProperty -SearchApplication $searchapp -Name $managedPropertyName -Type $type
    write-host "Created new managed property named $($managedPropertyName)."
  }
  
  $mm = Get-SPEnterpriseSearchMetadataMapping -SearchApplication $searchapp -CrawledProperty $cp -ManagedProperty $mp
  if ($mm) {
    write-host "Metadata mapping between properties $($managedPropertyName) and $($crawledPropertyName) already exist."
  }
  else {
    $mm = New-SPEnterpriseSearchMetadataMapping -SearchApplication $searchapp -CrawledProperty $cp -ManagedProperty $mp
    write-host "Created new metadata mapping between properties $($managedPropertyName) and $($crawledPropertyName)."
  }
}

Function RemoveManagedProperty([string]$managedPropertyName)
{
  $ErrorActionPreference = "SilentlyContinue"
  $mp = Get-SPEnterpriseSearchMetadataManagedProperty -SearchApplication $searchapp -Identity $managedPropertyName
  if ($mp) {
    $mp | Remove-SPEnterpriseSearchMetadataManagedProperty
    write-host "Removed managed property named $($managedPropertyName)."
  }
  else {
    write-host "Managed property named $($managedPropertyName) was not found."
  }
}
Function RemoveMetadataMapping([string]$managedPropertyName, [string]$crawledPropertyName)
{
  $ErrorActionPreference = "SilentlyContinue"
  $mp = Get-SPEnterpriseSearchMetadataManagedProperty -SearchApplication $searchapp -Identity $managedPropertyName
  $cp = Get-SPEnterpriseSearchMetadataCrawledProperty -SearchApplication $searchapp -Category $category -Name $crawledPropertyName
  if ($cp -AND $mp) {
    $mm = Get-SPEnterpriseSearchMetadataMapping -SearchApplication $searchapp -CrawledProperty $cp -ManagedProperty $mp
    if ($mm) {
      $mm | Remove-SPEnterpriseSearchMetadataMapping
      write-host "Removed metadata mapping between properties $($managedPropertyName) and $($crawledPropertyName)."
    }
    else {
      write-host "Metadata mapping between properties $($managedPropertyName) and $($crawledPropertyName) was not found."
    }
  }
}

Function CreateMetadataMappings
{
  Write-Host -foregroundcolor yellow Creating metadata mappings.
  $searchapp = Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp
  # A horrible kludge to deal with localization applied to where no localization should ever go.
  $category = Get-SPEnterpriseSearchMetadataCategory -SearchApplication $searchapp Identity "Business Data" -ErrorAction SilentlyContinue
  if (!$category)
  {
    Write-Host -foregroundcolor Yellow Category `"Business Data`" was not found. This may be due to SharePoint being a localized version. Please see the list below for the available categories `
    and type in the translated equivalent for `"Business Data`":
    $categories = Get-SPEnterpriseSearchMetadataCategory -SearchApplication $searchapp
    $categories | Foreach-Object {$_.Name}
    $category = Read-Host -prompt "Translated category: "
  }
  # Horrible kludge over. You can open your eyes now.
  CreateMetadataMappingIfNeeded "Type" "Entity.Type" 1 31
  CreateMetadataMappingIfNeeded "Title" "Entity.Name" 1 31
  CreateMetadataMappingIfNeeded "Author" "Entity.Author" 1 31
  CreateMetadataMappingIfNeeded "Description" "Entity.Description" 1 31
  CreateMetadataMappingIfNeeded "Symbol" "Entity.Symbol" 1 31
  CreateMetadataMappingIfNeeded "Context" "Entity.Context" 1 31
  CreateMetadataMappingIfNeeded "Context" "EmbeddedFile.Context" 1 31
  CreateMetadataMappingIfNeeded "Context" "EmbeddedFileDocx.Context" 1 31
  CreateMetadataMappingIfNeeded "Context" "EmbeddedFileXlsx.Context" 1 31
  CreateMetadataMappingIfNeeded "Context" "EmbeddedFilePptx.Context" 1 31
  CreateMetadataMappingIfNeeded "Context" "EmbeddedFilePdf.Context" 1 31
  CreateMetadataMappingIfNeeded "Filename" "EmbeddedFile.Name" 1 31
  CreateMetadataMappingIfNeeded "Filename" "EmbeddedFileDocx.Name" 1 31
  CreateMetadataMappingIfNeeded "Filename" "EmbeddedFileXlsx.Name" 1 31
  CreateMetadataMappingIfNeeded "Filename" "EmbeddedFilePptx.Name" 1 31
  CreateMetadataMappingIfNeeded "Filename" "EmbeddedFilePdf.Name" 1 31
  CreateMetadataMappingIfNeeded "FileExtension" "EmbeddedFile.Extension" 1 31
  CreateMetadataMappingIfNeeded "FileExtension" "EmbeddedFileDocx.Extension" 1 31
  CreateMetadataMappingIfNeeded "FileExtension" "EmbeddedFileXlsx.Extension" 1 31
  CreateMetadataMappingIfNeeded "FileExtension" "EmbeddedFilePptx.Extension" 1 31
  CreateMetadataMappingIfNeeded "FileExtension" "EmbeddedFilePdf.Extension" 1 31
  CreateMetadataMappingIfNeeded "DetailsUrl" "Entity.DetailsUrl" 1 31
  CreateMetadataMappingIfNeeded "DetailsUrl" "EmbeddedFile.DetailsUrl" 1 31
  CreateMetadataMappingIfNeeded "DetailsUrl" "EmbeddedFileDocx.DetailsUrl" 1 31
  CreateMetadataMappingIfNeeded "DetailsUrl" "EmbeddedFileXlsx.DetailsUrl" 1 31
  CreateMetadataMappingIfNeeded "DetailsUrl" "EmbeddedFilePptx.DetailsUrl" 1 31
  CreateMetadataMappingIfNeeded "DetailsUrl" "EmbeddedFilePdf.DetailsUrl" 1 31
  CreateMetadataMappingIfNeeded "ChartUrl" "Entity.ChartUrl" 1 31
  CreateMetadataMappingIfNeeded "EditUrl" "Entity.EditUrl" 1 31
  CreateMetadataMappingIfNeeded "IconUrl" "Entity.IconUrl" 1 31
  CreateMetadataMappingIfNeeded "AttachedText" "Entity.AttachedText" 1 31
}
Function RemoveManagedProperties
{
  $searchapp = Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp
  $category = Get-SPEnterpriseSearchMetadataCategory -SearchApplication $searchapp Identity "Business Data"
  RemoveManagedProperty "Type"
  RemoveManagedProperty "Context"
  RemoveManagedProperty "DetailsUrl"
  RemoveManagedProperty "IconUrl"
  RemoveManagedProperty "Symbol"
  RemoveManagedProperty "ChartUrl"
  RemoveManagedProperty "EditUrl"
  RemoveManagedProperty "MimeType"
  RemoveManagedProperty "AttachedText"
}

Function RemoveMetadataMappings
{
  $searchapp = Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp
  $category = Get-SPEnterpriseSearchMetadataCategory -SearchApplication $searchapp Identity "Business Data"
  RemoveMetadataMapping "Type" "Entity.Type"
  RemoveMetadataMapping "Title" "Entity.Name"
  RemoveMetadataMapping "Author" "Entity.Author"
  RemoveMetadataMapping "Description" "Entity.Description"
  RemoveMetadataMapping "Symbol" "Entity.Symbol"
  RemoveMetadataMapping "Context" "Entity.Context"
  RemoveMetadataMapping "Context" "EmbeddedFile.Context"
  RemoveMetadataMapping "Context" "EmbeddedFileDocx.Context"
  RemoveMetadataMapping "Context" "EmbeddedFileXlsx.Context"
  RemoveMetadataMapping "Context" "EmbeddedFilePptx.Context"
  RemoveMetadataMapping "Context" "EmbeddedFilePdf.Context"
  RemoveMetadataMapping "Filename" "EmbeddedFile.Name"
  RemoveMetadataMapping "Filename" "EmbeddedFileDocx.Name"
  RemoveMetadataMapping "Filename" "EmbeddedFileXlsx.Name"
  RemoveMetadataMapping "Filename" "EmbeddedFilePptx.Name"
  RemoveMetadataMapping "Filename" "EmbeddedFilePdf.Name"
  RemoveMetadataMapping "FileExtension" "EmbeddedFile.Extension"
  RemoveMetadataMapping "FileExtension" "EmbeddedFileDocx.Extension"
  RemoveMetadataMapping "FileExtension" "EmbeddedFileXlsx.Extension"
  RemoveMetadataMapping "FileExtension" "EmbeddedFilePptx.Extension"
  RemoveMetadataMapping "FileExtension" "EmbeddedFilePdf.Extension"
  RemoveMetadataMapping "DetailsUrl" "Entity.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFile.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFileDocx.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFileXlsx.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFilePptx.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFilePdf.DetailsUrl"
  RemoveMetadataMapping "ChartUrl" "Entity.ChartUrl"
  RemoveMetadataMapping "EditUrl" "Entity.EditUrl"
  RemoveMetadataMapping "IconUrl" "Entity.IconUrl"
  RemoveMetadataMapping "Filename" "Entity.FileName"
  RemoveMetadataMapping "MimeType" "Entity.MimeType"
  RemoveMetadataMapping "AttachedText" "Entity.AttachedText"
  RemoveManagedProperties
}

Function CheckDependencies
{
  $searchService = Get-SPServiceApplication -Name $targetSearchApp -ErrorAction SilentlyContinue
  $bdcService = Get-SPServiceApplication | Where {$_.TypeName -eq "Business Data Connectivity Service Application"} -ErrorAction SilentlyContinue
  
  if (!$searchService)
  { 
    $answer = Read-Host -Prompt "$targetSearchApp was not found. Installation will most likely fail. Do you want to continue? y/n" 
    if ($answer -eq "n")
    {
      exit
    }
  }
  elseif ($searchService.status -ne "Online") 
  {
    $answer = Read-Host -Prompt "$targetSearchApp is not active. Installation will most likely fail. Do you want to continue? y/n" 
    if ($answer -eq "n")
    {
      exit
    }
  }
  
  if (!$bdcService) 
  { 
    $answer = Read-Host -Prompt "Business Data Connectivity Service was not found. Installation will most likely fail. Do you want to continue? y/n" 
    if ($answer -eq "n")
    {
      exit
    }
  }
  elseif ($bdcService.status -ne "Online") 
  {
    $answer = Read-Host -Prompt "Business Data Connectivity Service is not active. Installation will most likely fail. Do you want to continue? y/n" 
    if ($answer -eq "n")
    {
      exit
    }
  }
  
  $currentUser = [Environment]::UserDomainName.ToString() + "\" + [Environment]::UserName.ToString()
  
  if ($currentUser -ne $owneralias)
  {
    $answer = Read-Host -Prompt "Current user is $currentUser while the configured owner is $owneralias. This mismatch may cause installation to fail, continue only if you are sure that the rights are sufficient for both users. Do you want to continue? y/n"
    if ($answer -eq "n")
    {
      exit
    }
  }

  $res = Get-SPManagedPath -WebApplication $rootUrl -Identity $siteRoot -ErrorAction SilentlyContinue
  if (!$res) 
  {
    $existingPaths = Get-SPManagedPath -WebApplication $rootUrl | Where {$_.type -eq "WildCardInclusion"}
    Write-Host -foregroundcolor Yellow "Managed path for $siteRoot could not be found. Do you want to create a managed path? `
    If you don't want to create a new path, answer n to exit setup and change Configuration.ps1 to use one of the following existing paths as the value for the siteRoot setting:"
    $existingPaths
    $answer = Read-Host -Prompt "Create new path (y/n)? "
    if ($answer -eq "y")
    {
       New-SPManagedPath -RelativeUrl $siteRoot -WebApplication $rootUrl
    }
    else 
    {
      exit
    }
  }
}

########## QPR Search Central setup script starts ##########

# Check if SharePoint snapin is loaded

Write-Host -foregroundcolor green Loading snap-ins...

Load-Snapins

Start-Service spadminv4

CheckDependencies

Write-Host -foregroundcolor green Creating crawl rule for QPR content...

$searchApplication = Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp;
$crawlRule = Get-SPEnterpriseSearchCrawlRule -SearchApplication $searchApplication | where { ($_.Path -eq $crawlRulePath) -and ($_.Type -eq "InclusionRule") };
if (!$crawlRule) {
  New-SPEnterpriseSearchCrawlRule -SearchApplication $searchApplication -Path $crawlRulePath -Type InclusionRule
}

$securityTrimmer = Get-SPEnterpriseSearchSecurityTrimmer -SearchApplication $searchApplication | where { ($_.Id -eq 1) -and ($_.CrawlRulePath -like "*" + $crawlRulePath + "*") -and ($_.TypeName -like "*Microsoft.Office.Server.Search.Connector.BDC.SPBDC.SPBdcSecurityTrimmer*") };
if (!$securityTrimmer) {
  New-SPEnterpriseSearchSecurityTrimmer -SearchApplication $searchApplication -TypeName "Microsoft.Office.Server.Search.Connector.BDC.SPBDC.SPBdcSecurityTrimmer, Microsoft.Office.Server.Search.Connector, Version=14.0.0.0, Culture=neutral, PublicKeyToken=71e9bce111e9429c" -RulePath $crawlRulePath -Id 1
}

Write-Host -foregroundcolor green Adding Qpr.BdcModel solution...

$solution = Get-SPSolution | where { $_.Name -eq "Qpr.BdcModel.wsp" };
if (!$solution) {
  $solution = Add-SPSolution (Resolve-Path('Qpr.BdcModel.wsp'))
}
if (!$solution.Deployed) {
  Install-SPSolution -Identity Qpr.BdcModel.wsp -GACDeployment
}

Write-Host -foregroundcolor green Adding Qpr.MasterPage solution...

$solution = Get-SPSolution | where { $_.Name -eq "Qpr.MasterPage.wsp" };
if (!$solution) {
  $solution = Add-SPSolution (Resolve-Path('Qpr.MasterPage.wsp'))
}
if (!$solution.Deployed) {
  Install-SPSolution -Identity Qpr.MasterPage.wsp -GACDeployment
}

Write-Host -foregroundcolor green Running admin jobs to deploy the solutions...

Stop-Service spadminv4
Start-SPAdminJob
Start-Service spadminv4

#do { Start-Sleep 2 } while (!((Get-SPSolution "Qpr.BdcModel.wsp").Deployed))
#do { Start-Sleep 2 } while (!((Get-SPSolution "Qpr.MasterPage.wsp").Deployed))

Write-Host -foregroundcolor green Activating the QPR BDC farm feature...

$webAppsFeatureId = $(Get-SPFeature -limit all | where {$_.displayname -eq "QPR.BdcModel_BDC Feature"}).Id 
Enable-SPFeature $webAppsFeatureId -Url $rootUrl -ErrorAction SilentlyContinue

# Create profile pages host site

Write-Host -foregroundcolor green Creating site $profilePages for profile page host...

New-SPSite -Url $profilePages -Name "Profile Pages" -OwnerAlias $owneralias -Template "STS#1" -ErrorAction SilentlyContinue

# Create search site

Write-Host -foregroundcolor green Creating site $searchSite ...

InitializeQPRSearchSite $searchSite "" $ownerAlias $webContentUrl

# Configure profile page host site

Write-Host -foregroundcolor green Configuring profile pages host...

$context = Get-SPSite $rootUrl | Get-SPServiceContext
$profileProvisioner = New-Object Microsoft.SharePoint.Portal.WebControls.ProfileProvisioner($context)
$res = $profileProvisioner.SetHostSiteUri($profilePages)

# Create profile pages

Write-Host -foregroundcolor green Creating profile pages...

$res = $profileProvisioner.CreateProfilePage("Qpr.BdcModel.Model", "EmbeddedFile")
$res = $profileProvisioner.CreateProfilePage("Qpr.BdcModel.Model", "EmbeddedFileDocx")
$res = $profileProvisioner.CreateProfilePage("Qpr.BdcModel.Model", "EmbeddedFileXlsx")
$res = $profileProvisioner.CreateProfilePage("Qpr.BdcModel.Model", "EmbeddedFilePptx")
$res = $profileProvisioner.CreateProfilePage("Qpr.BdcModel.Model", "EmbeddedFilePdf")
$res = $profileProvisioner.CreateProfilePage("Qpr.BdcModel.Model", "Entity")

# Grant permissions to the object and metadata store for the BDC model

Write-Host -foregroundcolor green Granting permissions...

$adminURL="HKLM:\SOFTWARE\Microsoft\Shared Tools\Web Server Extensions\14.0\WSS\"

$key = Get-Item $adminURL 

$adminurl = $key.GetValue("CentralAdministrationURL")

Write-Host -foregroundcolor green Updating BCS permissions 

$obj = Get-SPBusinessDataCatalogMetadataObject -BdcObjectType Catalog -ServiceContext $adminurl

$right = (1+2+0x10+4)

$claim = New-SPClaimsPrincipal -Identity $crawlingAccount -IdentityType WindowsSamAccountName
  
Grant-SPBusinessDataCatalogMetadataObject -Identity $obj -Principal $claim  -Right $right

Copy-SPBusinessDataCatalogAclToChildren -MetadataObject $obj

$claim = New-SPClaimsPrincipal -Identity 'c:0!.s|windows' -IdentityType EncodedClaim
  
Grant-SPBusinessDataCatalogMetadataObject -Identity $obj -Principal $claim  -Right $right

Copy-SPBusinessDataCatalogAclToChildren -MetadataObject $obj

AddAccountToAllSites $searchSite $searchUsersGroup "Read"
AddAccountToAllSites $profilePages $searchUsersGroup "Read"

Write-Host -foregroundcolor green Granting permissions complete

# Create BDC content source

Write-Host -foregroundcolor green Creating BDC content source

$contentSource = New-SPEnterpriseSearchCrawlContentSource -SearchApplication (Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp) `
-Type business -name "QPR BDC Content Source" -LOBSystemSet "main" `
-startaddress "bdc3://qpr_model_main/Default/00000000%252D0000%252D0000%252D0000%252D000000000000/Qpr%252EModel/Main&s_ce=048440004080g10020o44"


CreateMetadataMappings
SetBdcModelProperties

# Activate QPR master page feature on the search site

Write-Host -foregroundcolor green Activating QPR master page feature...

$webAppsFeatureId = $(Get-SPFeature -limit all | where {$_.displayname -eq "QPR.MasterPage_QPRSiteContents"}).Id 
$searchSiteCollection = Get-SPSite -Identity $searchSite 
Enable-SPFeature $webAppsFeatureId -Url $searchSiteCollection.URL -ErrorAction SilentlyContinue

# Set search site to use the QPR Master Page
 
Write-Host -foregroundcolor green Configuring the search site to use QPR master page... 
 
$site = get-spsite -identity $searchSite
$web = $site.OpenWeb()
$web.CustomMasterUrl = $searchSitePath + "/_catalogs/masterpage/Qpr.master"
$web.update()
$web.dispose() 

Write-Host -foregroundcolor green Performing full crawl. Please wait.

$contentSource.StartFullCrawl()

Write-Host -foregroundcolor green QPR Search Central configuration finished. Now update QPR_Servers.ini to contain $searchSite as the value for the SearchServerLocation key in the [WS Settings]-section.